Match = Core.class(Shape)

function Match:init(id, boxes)
	self.id = id
	self.boxes = boxes

	self:setLineStyle(3, 0x000000)
	self:setFillStyle(Shape.SOLID, 0xff0000, 0.5)
	self:beginPath()
	self:moveTo(0, 0)
	self:lineTo(100, 0)
	self:lineTo(100, 25)
	self:lineTo(0, 25)
	self:closePath()
	self:endPath()
	self:setPosition(math.random(0, 240 - 50), math.random(0, 340 - 50))

	-- Create the label and attach it to the shape
    local label = TextField.new(nil, id)
    label:setPosition(self:getWidth()/2 - label:getWidth()/2, self:getHeight()/2 + label:getHeight()/2)
    self:addChild(label)

	stage:addChild(self) -- add everything to the stage

	self.isFocus = false
	self.enabled = true

    self:addEventListener(Event.MOUSE_DOWN, self.onMouseDown, self)
    self:addEventListener(Event.MOUSE_MOVE, self.onMouseMove, self)
    self:addEventListener(Event.MOUSE_UP, self.onMouseUp, self)

	return self
end

function Match:onMouseDown(event)
	if not self.enabled then return end
	if self:hitTestPoint(event.x, event.y) then
		self.isFocus = true		
		event:stopPropagation()
		
		self.x0 = event.x
        self.y0 = event.y
	end
end

function Match:onMouseMove(event)
	if not self.enabled then return end
    if self.isFocus then
        event:stopPropagation()
	
        local dx = event.x - self.x0
        local dy = event.y - self.y0
		
        self:setX(self:getX() + dx)
        self:setY(self:getY() + dy)
		
        self.x0 = event.x
        self.y0 = event.y
    end
end

function Match:onMouseUp(event)
	if not self.enabled then return end

	-- Return true if point is inside rect
	local function pointinrect(x1, y1, x2, y2, w, h)
		return x1 >= x2 and y1 >= y2 and x1 <= x2 + w and y1 <= y2 + h
	end

    if self.isFocus then
		self.isFocus = false
        event:stopPropagation()

		local boxes = self.boxes -- Check each box against this match
		for i = 1, #boxes do
			local box = boxes[i]
			local x, y, w, h = box:getBounds(stage)
			if pointinrect(self.x0, self.y0, x, y, w, h) then
				if box.id == self.id then
				
					print("Match Dropped in box with same id")
					
					self.enabled = false
					self:removeEventListener(Event.MOUSE_DOWN, self.onMouseDown, self) -- remove event listeners
					self:removeEventListener(Event.MOUSE_MOVE, self.onMouseMove, self)
					self:removeEventListener(Event.MOUSE_UP, self.onMouseUp, self)
					self:removeChildAt(1) -- remove label
					self:clear() -- get rid of all shape stuff
					self:removeFromParent() -- remove from parent (stage)
				else

					print("Match dropped into wrong box")

				end
			end
		end
    end
end
